@@ -19,7 +19,6 @@ local lua_util = require "lua_util"
 local logger = require "rspamd_logger"
 local lua_redis = require "lua_redis"
 local dmarc_common = require "plugins/dmarc"
-local lupa = require "lupa"
 local rspamd_mempool = require "rspamd_mempool"
 local rspamd_url = require "rspamd_url"
 local rspamd_text = require "rspamd_text"
@@ -46,9 +45,9 @@ parser:flag "-n --no-opt"
       :description "Do not reset reporting data/send reports"
 
 parser:argument "date"
-       :description "Date to process (today by default)"
-       :argname "<YYYYMMDD>"
-       :args "*"
+      :description "Date to process (today by default)"
+      :argname "<YYYYMMDD>"
+      :args "*"
 parser:option "-b --batch-size"
       :description "Send reports in batches up to <batch-size> messages"
       :argname "<number>"
@@ -102,14 +101,14 @@ local redis_attrs = {
 local pool
 
 local function load_config(opts)
-  local _r,err = rspamd_config:load_ucl(opts['config'])
+  local _r, err = rspamd_config:load_ucl(opts['config'])
 
   if not _r then
     logger.errx('cannot parse %s: %s', opts['config'], err)
     os.exit(1)
   end
 
-  _r,err = rspamd_config:parse_rcl({'logging', 'worker'})
+  _r, err = rspamd_config:parse_rcl({ 'logging', 'worker' })
   if not _r then
     logger.errx('cannot process %s: %s', opts['config'], err)
     os.exit(1)
@@ -118,11 +117,9 @@ end
 
 -- Concat elements using redis_keys.join_char
 local function redis_prefix(...)
-  return table.concat({...}, dmarc_settings.reporting.redis_keys.join_char)
+  return table.concat({ ... }, dmarc_settings.reporting.redis_keys.join_char)
 end
 
-
-
 local function get_rua(rep_key)
   local parts = lua_util.str_split(rep_key, dmarc_settings.reporting.redis_keys.join_char)
 
@@ -144,8 +141,8 @@ local function get_domain(rep_key)
 end
 
 local function gen_uuid()
-  local template ='xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'
-  return string.gsub(template, '[xy]', function (c)
+  local template = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'
+  return string.gsub(template, '[xy]', function(c)
     local v = (c == 'x') and math.random(0, 0xf) or math.random(8, 0xb)
     return string.format('%x', v)
   end)
@@ -159,7 +156,7 @@ local function gen_xml_grammar()
   local quot = lpeg.P('"') / '&quot;'
   local apos = lpeg.P("'") / '&apos;'
   local special = lt + gt + amp + quot + apos
-  local grammar = lpeg.Cs((special + 1)^0)
+  local grammar = lpeg.Cs((special + 1) ^ 0)
   return grammar
 end
 
@@ -178,8 +175,6 @@ local function escape_xml(input)
 
   return ''
 end
--- Enable xml escaping in lupa templates
-lupa.filters.escape_xml = escape_xml
 
 -- Creates report XML header
 local function report_header(reporting_domain, report_start, report_end, domain_policy)
@@ -213,7 +208,10 @@ local function report_header(reporting_domain, report_start, report_end, domain_
     report_end = report_end,
     domain_policy = domain_policy,
     reporting_domain = reporting_domain,
-  }, true)
+  }, true, false,
+      {
+        escape_xml = escape_xml
+      })
 end
 
 -- Generate xml entry for a preprocessed redis row
@@ -250,7 +248,10 @@ local function entry_to_xml(data)
   </auth_results>
 </record>
 ]]
-  return lua_util.jinja_template(xml_template, {data = data}, true)
+  return lua_util.jinja_template(xml_template, { data = data }, true,
+      false, {
+        escape_xml = escape_xml
+      })
 end
 
 -- Process a report entry stored in Redis splitting it to a lua table
@@ -273,7 +274,7 @@ local function process_report_entry(data, score)
     if dkim_data and dkim_data ~= '' then
       local dkim_elts = lua_util.str_split(dkim_data, '|')
       for _, d in ipairs(dkim_elts) do
-        table.insert(row.dkim_results, {domain = d, result = result})
+        table.insert(row.dkim_results, { domain = d, result = result })
       end
     end
   end
@@ -291,12 +292,13 @@ local function process_rua(dmarc_domain, rua)
 
   -- Remove size limitation, as we don't care about them
   local addrs = {}
-  for _,a in ipairs(parts) do
-    local u = rspamd_url.create(pool, a:gsub('!%d+[kmg]?$', ''))
-    if u then
+  for _, rua_part in ipairs(parts) do
+    local u = rspamd_url.create(pool, rua_part:gsub('!%d+[kmg]?$', ''))
+    local u2 = rspamd_url.create(pool, dmarc_domain)
+    if u and (u:get_protocol() or '') == 'mailto' and u:get_user() then
       -- Check each address for sanity
-      if dmarc_domain == u:get_tld() or dmarc_domain == u:get_host() then
-        -- Same domain - always include
+      if u:get_tld() == u2:get_tld() then
+        -- Same eSLD - always include
         table.insert(addrs, u)
       else
         -- We need to check authority
@@ -310,10 +312,10 @@ local function process_rua(dmarc_domain, rua)
         })
 
         if not is_ok then
-          logger.errx('cannot resolve %s: %s; exclude %s', resolve_str, results, a)
+          logger.errx('cannot resolve %s: %s; exclude %s', resolve_str, results, rua_part)
         else
           local found = false
-          for _,t in ipairs(results) do
+          for _, t in ipairs(results) do
             if string.match(t, 'v=DMARC1') then
               found = true
               break
@@ -328,6 +330,8 @@ local function process_rua(dmarc_domain, rua)
           end
         end
       end
+    else
+      logger.errx('invalid rua url: "%s""', tostring(u or 'null'))
     end
   end
 
@@ -341,26 +345,23 @@ end
 -- Validate reporting domain, extracting rua and checking 3rd party report domains
 -- This function returns a full dmarc record processed + rua as a list of url objects
 local function validate_reporting_domain(reporting_domain)
-  -- Now check the domain policy
-  -- DMARC domain is a esld for the reporting domain
-  local dmarc_domain = rspamd_util.get_tld(reporting_domain)
   local is_ok, results = rspamd_dns.request({
     config = rspamd_config,
     session = rspamadm_session,
     type = 'txt',
-    name = '_dmarc.' .. dmarc_domain ,
+    name = '_dmarc.' .. reporting_domain,
   })
 
   if not is_ok or not results then
-    logger.errx('cannot resolve _dmarc.%s: %s', dmarc_domain, results)
+    logger.errx('cannot resolve _dmarc.%s: %s', reporting_domain, results)
     return nil
   end
 
-  for _,r in ipairs(results) do
-    local processed,rec = dmarc_common.dmarc_check_record(rspamd_config, r, false)
+  for _, r in ipairs(results) do
+    local processed, rec = dmarc_common.dmarc_check_record(rspamd_config, r, false)
     if processed and rec.rua then
       -- We need to check or alter rua if needed
-      local processed_rua = process_rua(dmarc_domain, rec.rua)
+      local processed_rua = process_rua(reporting_domain, rec.rua)
       if processed_rua then
         rec = rec.raw_elts
         rec.rua = processed_rua
@@ -383,7 +384,7 @@ end
 -- Returns a list of recipients from a table as a string processing elements if needed
 local function rcpt_list(tbl, func)
   local res = {}
-  for _,r in ipairs(tbl) do
+  for _, r in ipairs(tbl) do
     if func then
       table.insert(res, func(r))
     else
@@ -429,16 +430,16 @@ local function send_reports_by_smtp(opts, reports, finish_cb)
     local nreports = math.min(#reports - cur_batch + 1, opts.batch_size)
     local next_start = cur_batch + nreports
     lua_util.debugm(N, 'send data for %s domains (from %s to %s)',
-        nreports, cur_batch, next_start-1)
+        nreports, cur_batch, next_start - 1)
     -- Shared across all closures
     local gen_args = {
       cont_func = send_data_in_batches,
       nreports = nreports,
       next_start = next_start
     }
-    for i=cur_batch,next_start-1 do
+    for i = cur_batch, next_start - 1 do
       local report = reports[i]
-      lua_smtp.sendmail({
+      local send_opts = {
         ev_base = rspamadm_ev_base,
         session = rspamadm_session,
         config = rspamd_config,
@@ -448,7 +449,9 @@ local function send_reports_by_smtp(opts, reports, finish_cb)
         from = report_settings.email,
         recipients = report.rcpts,
         helo = report_settings.helo or 'rspamd.localhost',
-      },
+      }
+
+      lua_smtp.sendmail(send_opts,
           report.message,
           gen_sendmail_cb(report, gen_args))
     end
@@ -471,7 +474,7 @@ local function prepare_report(opts, start_time, end_time, rep_key)
   end
 
   local ret, results = lua_redis.request(redis_params, redis_attrs,
-      {'EXISTS', rep_key})
+      { 'EXISTS', rep_key })
 
   if not ret or not results or results == 0 then
     return nil
@@ -480,7 +483,7 @@ local function prepare_report(opts, start_time, end_time, rep_key)
   -- Rename report key to avoid races
   if not opts.no_opt then
     lua_redis.request(redis_params, redis_attrs,
-        {'RENAME', rep_key, rep_key .. '_processing'})
+        { 'RENAME', rep_key, rep_key .. '_processing' })
     rep_key = rep_key .. '_processing'
   end
 
@@ -490,7 +493,7 @@ local function prepare_report(opts, start_time, end_time, rep_key)
   if not dmarc_record then
     if not opts.no_opt then
       lua_redis.request(redis_params, redis_attrs,
-          {'DEL', rep_key})
+          { 'DEL', rep_key })
     end
     logger.messagex('Cannot process reports for domain %s; invalid dmarc record', reporting_domain)
     return nil
@@ -498,11 +501,11 @@ local function prepare_report(opts, start_time, end_time, rep_key)
 
   -- Get all reports for a domain
   ret, results = lua_redis.request(redis_params, redis_attrs,
-      {'ZRANGE', rep_key, '0', '-1', 'WITHSCORES'})
+      { 'ZRANGE', rep_key, '0', '-1', 'WITHSCORES' })
   local report_entries = {}
   table.insert(report_entries,
       report_header(reporting_domain, start_time, end_time, dmarc_record))
-  for i=1,#results,2 do
+  for i = 1, #results, 2 do
     local xml_record = entry_to_xml(process_report_entry(results[i], results[i + 1]))
     table.insert(report_entries, xml_record)
   end
@@ -534,28 +537,32 @@ local function prepare_report(opts, start_time, end_time, rep_key)
     message_id = rspamd_util.random_hex(16) .. '@' .. report_settings.msgid_from,
     report_start = start_time,
     report_end = end_time
-  }, true)
+  }, true,
+      false, {
+        escape_xml = escape_xml
+      })
   local rfooter = lua_util.jinja_template(report_footer, {
     uuid = uuid,
-  }, true)
-  local message = rspamd_text.fromtable{
+  }, true, false, {
+    escape_xml = escape_xml
+  })
+  local message = rspamd_text.fromtable {
     (rhead:gsub("\n", "\r\n")),
     rspamd_util.encode_base64(rspamd_util.gzip_compress(xml_to_compress), 73),
     rfooter:gsub("\n", "\r\n"),
   }
 
-
   lua_util.debugm(N, 'got final message: %s', message)
 
   if not opts.no_opt then
     lua_redis.request(redis_params, redis_attrs,
-        {'DEL', rep_key})
+        { 'DEL', rep_key })
   end
 
   local report_rcpts = lua_util.str_split(rcpt_string, ',')
 
   if report_settings.bcc_addrs then
-    for _,b in ipairs(report_settings.bcc_addrs) do
+    for _, b in ipairs(report_settings.bcc_addrs) do
       table.insert(report_rcpts, b)
     end
   end
@@ -570,7 +577,7 @@ end
 local function process_report_date(opts, start_time, end_time, date)
   local idx_key = redis_prefix(dmarc_settings.reporting.redis_keys.index_prefix, date)
   local ret, results = lua_redis.request(redis_params, redis_attrs,
-      {'EXISTS', idx_key})
+      { 'EXISTS', idx_key })
 
   if not ret or not results or results == 0 then
     logger.messagex('No reports for %s', date)
@@ -580,24 +587,24 @@ local function process_report_date(opts, start_time, end_time, date)
   -- Rename index key to avoid races
   if not opts.no_opt then
     lua_redis.request(redis_params, redis_attrs,
-        {'RENAME', idx_key, idx_key .. '_processing'})
+        { 'RENAME', idx_key, idx_key .. '_processing' })
     idx_key = idx_key .. '_processing'
   end
   ret, results = lua_redis.request(redis_params, redis_attrs,
-      {'SMEMBERS', idx_key})
+      { 'SMEMBERS', idx_key })
 
   if not ret or not results then
     -- Remove bad key
     if not opts.no_opt then
       lua_redis.request(redis_params, redis_attrs,
-          {'DEL', idx_key})
+          { 'DEL', idx_key })
     end
     logger.messagex('Cannot get reports for %s', date)
     return {}
   end
 
   local reports = {}
-  for _,rep in ipairs(results) do
+  for _, rep in ipairs(results) do
     local report = prepare_report(opts, start_time, end_time, rep)
 
     if report then
@@ -610,7 +617,7 @@ local function process_report_date(opts, start_time, end_time, date)
   -- Remove processed key
   if not opts.no_opt then
     lua_redis.request(redis_params, redis_attrs,
-        {'DEL', idx_key})
+        { 'DEL', idx_key })
   end
 
   return reports
@@ -619,7 +626,7 @@ end
 
 -- Returns a day before today at 00:00 as unix seconds
 local function yesterday_midnight()
-  local piecewise_time = os.date("!*t")
+  local piecewise_time = os.date("*t")
   piecewise_time.day = piecewise_time.day - 1 -- Lua allows negative values here
   piecewise_time.hour = 0
   piecewise_time.sec = 0
@@ -629,7 +636,7 @@ end
 
 -- Returns today time at 00:00 as unix seconds
 local function today_midnight()
-  local piecewise_time = os.date("!*t")
+  local piecewise_time = os.date("*t")
   piecewise_time.hour = 0
   piecewise_time.sec = 0
   piecewise_time.min = 0
@@ -665,14 +672,14 @@ local function handler(args)
     os.exit(1)
   end
 
-  for _, e in ipairs({'email', 'domain', 'org_name'}) do
+  for _, e in ipairs({ 'email', 'domain', 'org_name' }) do
     if not dmarc_settings.reporting[e] then
       logger.errx('Missing required setting: dmarc.reporting.%s', e)
       return
     end
   end
 
-  local ret,results = lua_redis.request(redis_params, redis_attrs, {
+  local ret, results = lua_redis.request(redis_params, redis_attrs, {
     'GET', 'rspamd_dmarc_last_collection'
   })
 
@@ -686,20 +693,20 @@ local function handler(args)
 
   if not opts.date or #opts.date == 0 then
     opts.date = {}
-    table.insert(opts.date, os.date('!%Y%m%d', yesterday_midnight()))
+    table.insert(opts.date, os.date('%Y%m%d', yesterday_midnight()))
   end
 
   local ndates = 0
   local nreports = 0
   local all_reports = {}
-  for _,date in ipairs(opts.date) do
+  for _, date in ipairs(opts.date) do
     lua_util.debugm(N, 'Process date %s', date)
     local reports_for_date = process_report_date(opts, start_time, start_collection, date)
     if #reports_for_date > 0 then
       ndates = ndates + 1
       nreports = nreports + #reports_for_date
 
-      for _,r in ipairs(reports_for_date) do
+      for _, r in ipairs(reports_for_date) do
         table.insert(all_reports, r)
       end
     end
@@ -714,8 +721,8 @@ local function handler(args)
             ndates, nreports, nsuccess, nfail)
       end
       lua_redis.request(redis_params, redis_attrs,
-          {'SETEX', 'rspamd_dmarc_last_collection', dmarc_settings.reporting.keys_expire * 2,
-           tostring(start_collection)})
+          { 'SETEX', 'rspamd_dmarc_last_collection', dmarc_settings.reporting.keys_expire * 2,
+            tostring(start_collection) })
     else
       logger.messagex('Reporting collection has finished %s dates processed, %s reports: %s completed, %s failed',
           ndates, nreports, nsuccess, nfail)
@@ -723,12 +730,16 @@ local function handler(args)
 
     pool:destroy()
   end
-  send_reports_by_smtp(opts, all_reports, finish_cb)
+  if not opts.no_opt then
+    send_reports_by_smtp(opts, all_reports, finish_cb)
+  else
+    logger.messagex('Skip sending mails due to -n / --no-opt option')
+  end
 end
 
 return {
   name = 'dmarc_report',
-  aliases = {'dmarc_reporting'},
+  aliases = { 'dmarc_reporting' },
   handler = handler,
   description = parser._description
-}
\ No newline at end of file
+}
